<?php

namespace FiloBlu\Maintenance\Observer\Backend\Backend;

class AuthUserLoginSuccess implements \Magento\Framework\Event\ObserverInterface
{

    const DEFAULT_SESSION_NAME_OF_FRONTEND = 'PHPSESSID';
    const ADMIN_ACCESS_CONFIG_PATH = 'filoblu_maintenance_section/general/admin_only';
    const ADMIN_SUBDOMAIN_CONFIG_PATH = 'filoblu_maintenance_section/general/admin_subdomain';
    const COOKIE_DURATION = 473040000; //15 years
    const COOKIE_NAME = 'filoblu_maintenance_admin_loggedin';
    protected $helper;
    protected $config;
    protected $session;
    protected $encryptor;

    /**
     * @var CookieManagerInterface
     */
    protected $cookieManager;
    /**
     * @var CookieMetadataFactory
     */
    protected $cookieMetadataFactory;

    public function __construct(
        \FiloBlu\Maintenance\Helper\Data $helper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Backend\Model\Auth\Session $session,
        \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager,
        \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory $cookieMetadataFactory,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor
    )
    {
        $this->helper = $helper;
        $this->config = $scopeConfig;
        $this->session = $session;
        $this->cookieManager = $cookieManager;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->encryptor = $encryptor;
    }

    /**
     * Drop a cookie with hashed session_id, to be read at frontend
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(
        \Magento\Framework\Event\Observer $observer
    )
    {

        if ($this->config->isSetFlag(self::ADMIN_ACCESS_CONFIG_PATH)) {
            $adminSubdomain = $this->config->getValue(self::ADMIN_SUBDOMAIN_CONFIG_PATH);
            $frontendDomain = str_replace($adminSubdomain, '', $this->session->getCookieDomain());
            $metadata = $this->cookieMetadataFactory
                ->createPublicCookieMetadata()
                ->setDuration(self::COOKIE_DURATION)
                ->setDomain($frontendDomain)
                ->setPath('/');
            $this->cookieManager
                ->setPublicCookie(self::COOKIE_NAME, $this->encryptor->hash($this->session->getSessionId()), $metadata);
        }

        return;
    }
}